--------------------------------------------------------------------------------
-- Copyright(C) 2005 by Xilinx, Inc. All rights reserved.
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you
-- a license to use this text/file solely for design, simulation,
-- implementation and creation of design files limited
-- to Xilinx devices or technologies. Use with non-Xilinx
-- devices or technologies is expressly prohibited and
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information
-- "as is" solely for use in developing programs and
-- solutions for Xilinx devices. By providing this design,
-- code, or information as one possible implementation of
-- this feature, application or standard, Xilinx is making no
-- representation that this implementation is free from any
-- claims of infringement. You are responsible for
-- obtaining any rights you may require for your implementation.
-- Xilinx expressly disclaims any warranty whatsoever with
-- respect to the adequacy of the implementation, including
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied
-- warranties of merchantability or fitness for a particular
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications are
-- expressly prohibited.
--
-- This copyright and support notice must be retained as part
-- of this text at all times. (c) Copyright 2005 Xilinx, Inc.
-- All rights reserved.
--------------------------------------------------------------------------------
--
-- Filename - cntr1_new.vhd
-- Author - P. Szanto
-- Creation - 27 June 2006
--
-- Description -
--
-- Bit summation (or in other words counting the "1"s) module for the new samples
-- Implemented as an adder tree
-- At the first stage ADDER_INS wide chunks of the input are processed,
-- then the results are added together with a pipelined adder tree
-- Except the first stage, are two-input
-- Latency is configuration dependent
--
--
-- $RCSfile: cntr1_new.vhd,v $ $Revision: 1.5 $ $Date: 2006/06/30 19:01:22 $
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

use work.rank2d_utils.ALL; 
use work.rank2d_latency.ALL; 

entity cntr1_new is
generic(
   TAP      : integer := 9;
   TAPb     : integer := 4;
   RST_VAL  : integer := 0;
   ADDER_INS: integer := 6
);
port(
   clk         : in std_logic;
   en          : in std_logic;
   data        : in std_logic_vector(TAP-1 downto 0);
   res         : out std_logic_vector(TAPb-1 downto 0)
);
end cntr1_new;

architecture rtl of cntr1_new is

constant CONST_0 : std_logic_vector(127 downto 0) := x"00000000000000000000000000000000";

constant cnt1_1stage : integer := MAX(2, DIV_CEIL_2PWR(TAP, ADDER_INS));
constant cnt1_stages : integer := GET_1CNT_STAGES(TAP, ADDER_INS);

type array_TAPdIN_by_TAPb is array (cnt1_1stage-1 downto 0) of std_logic_vector(TAPb-1 downto 0);
type array_1cnt_stages   is array (cnt1_stages-1 downto 0) of array_TAPdIN_by_TAPb;

signal one_cntr : array_1cnt_stages;

begin

-- bit sum for TAP wide input
-- output is TAPb bit wide
-- at the first stage input is partitioned into ADDER_INS bit wide parts
-- and these are summed.
-- all other stages sum two outputs from the previous stage


GEN_1CNT_L0:
for IST in 0 to cnt1_stages-1 generate
   GEN_1cnt_0:
   if (IST = 0) generate
      process(clk)
      variable cntr_regs : array_TAPdIN_by_TAPb;
      begin
         if (clk'event and clk='1') then
            if (en='1') then
            for IR in 0 to cnt1_1stage-1 loop
               cntr_regs(IR) := CONST_0(TAPb-1 downto 0);
            end loop;
            for IB in 0 to TAP-1 loop
               cntr_regs(IB/ADDER_INS) := cntr_regs(IB/ADDER_INS) + data(IB);
            end loop;
            for IR in 0 to cnt1_1stage-1 loop
               one_cntr(0)(IR) <= cntr_regs(IR);
            end loop;
            end if;
         end if;
      end process;
   end generate;
   GEN_1cnt_last:
   if ((IST = (cnt1_stages-1)) AND (IST /= 0)) generate
      process(clk)
      begin
      if (clk'event and clk='1') then
         if (en='1') then
            one_cntr(IST)(0) <= one_cntr(IST-1)(0) + one_cntr(IST-1)(1);
         end if;
      end if;
      end process;
   end generate;
   GEN_1cnt_others:
   if ((IST /= 0) AND (IST /= (cnt1_stages-1))) generate
      process(clk)
      begin
         if (clk'event and clk='1') then
            if (en='1') then
            for IR in 0 to DIV_CEIL(cnt1_1stage, (2**IST))-1 loop
               one_cntr(IST)(IR) <= one_cntr(IST-1)(2*IR) + one_cntr(IST-1)(2*IR+1);
            end loop;
            end if;
         end if;
      end process;
   end generate;
end generate;

res <= one_cntr(cnt1_stages-1)(0);

end rtl;
