--------------------------------------------------------------------------------
-- Copyright(C) 2005 by Xilinx, Inc. All rights reserved.
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you
-- a license to use this text/file solely for design, simulation,
-- implementation and creation of design files limited
-- to Xilinx devices or technologies. Use with non-Xilinx
-- devices or technologies is expressly prohibited and
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information
-- "as is" solely for use in developing programs and
-- solutions for Xilinx devices. By providing this design,
-- code, or information as one possible implementation of
-- this feature, application or standard, Xilinx is making no
-- representation that this implementation is free from any
-- claims of infringement. You are responsible for
-- obtaining any rights you may require for your implementation.
-- Xilinx expressly disclaims any warranty whatsoever with
-- respect to the adequacy of the implementation, including
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied
-- warranties of merchantability or fitness for a particular
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications are
-- expressly prohibited.
--
-- This copyright and support notice must be retained as part
-- of this text at all times. (c) Copyright 2005 Xilinx, Inc.
-- All rights reserved.
--------------------------------------------------------------------------------
--
-- Filename - cntr1_old.vhd
-- Author - P. Szanto
-- Creation - 27 June 2006
--
-- Description -
--
-- Bit summation (or in other words counting the "1"s) module for the old samples
-- The number of "1"s in the comparison results of the NI discarded samples
-- and the NI new samples are counted with adder tree (cntr1_new)
-- Based on these results a modification value is generated which has a range (-NI...N),
-- and this value is added to the incoming CNTR value
-- Latency is configuration dependent
--
--
-- $RCSfile: cntr1_old.vhd,v $ $Revision: 1.4 $ $Date: 2006/06/30 19:01:23 $
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


use work.rank2d_utils.ALL;
use work.rank2d_latency.ALL; 

entity cntr1_old is

generic(
   TAP      : integer := 9;
   NI       : integer := 1;
   TAPb     : integer := 6;
   RST_VAL  : integer := 0;
   ADDER_INS: integer := 4
);

port(
   clk          : in std_logic;
   rst          : in std_logic;
   en           : in std_logic;
   data_old     : in std_logic_vector(NI-1 downto 0);
   data_new     : in std_logic_vector(NI-1 downto 0);
   cntr_in      : in std_logic_vector(TAPb-1 downto 0);
   res          : out std_logic_vector(TAPb-1 downto 0)
);
end cntr1_old;

architecture rtl of cntr1_old is

constant cnt1_full_stages : integer := GET_1CNT_STAGES(TAP, ADDER_INS);
constant cnt1_stages      : integer := GET_1CNT_STAGES(NI, ADDER_INS);
constant modval_stages    : integer := 1;
constant moddl_stages     : integer := cnt1_full_stages - cnt1_stages;

type array_moddl_by_TAPb is array (moddl_stages-1 downto 0) of std_logic_vector(TAPb-1 downto 0);
signal cnt1_new : std_logic_vector(TAPb-1 downto 0);
signal cnt1_old : std_logic_vector(TAPb-1 downto 0);
signal mod_dl   : array_moddl_by_TAPb;
signal one_cntr : std_logic_vector(TAPb-1 downto 0);

begin


-- counting '1's in the new/old inputs
count_new: entity work.cntr1_new(rtl)
generic map(
   TAP       => NI,
   TAPb      => TAPb,
   RST_VAL   => 0,
   ADDER_INS => ADDER_INS
)
port map(
   clk         => clk,
   en          => en,
   data        => data_new,
   res         => cnt1_new
);



count_old: entity work.cntr1_new(rtl)
generic map(
   TAP       => NI,
   TAPb      => TAPb,
   RST_VAL   => 0,
   ADDER_INS => ADDER_INS
)
port map(
   clk         => clk,
   en          => en,
   data        => data_old,
   res         => cnt1_old
);



GEN_MOD_VAL:
for IST in 0 to moddl_stages-1 generate
   GEN_stage0:
   if (IST = 0) generate
      process(clk)
      begin
         if (clk'event and clk='1') then
            if (en='1') then
               mod_dl(IST) <= cnt1_new - cnt1_old;
            end if;
         end if;
      end process;
   end generate;
   GEN_others:
   if (IST /= 0) generate
      process(clk)
      begin
         if (clk'event and clk='1') then
            if (en='1') then
               mod_dl(IST) <= mod_dl(IST-1);
            end if;
         end if;
      end process;
   end generate;
end generate;

process(clk)
begin
   if (clk'event and clk='1') then
      if (en='1') then
         one_cntr <= cntr_in + mod_dl(moddl_stages-1);
      end if;
   end if;
end process;

res <= one_cntr;


end rtl;

