--------------------------------------------------------------------------------
-- Copyright(C) 2005 by Xilinx, Inc. All rights reserved.
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you
-- a license to use this text/file solely for design, simulation,
-- implementation and creation of design files limited
-- to Xilinx devices or technologies. Use with non-Xilinx
-- devices or technologies is expressly prohibited and
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information
-- "as is" solely for use in developing programs and
-- solutions for Xilinx devices. By providing this design,
-- code, or information as one possible implementation of
-- this feature, application or standard, Xilinx is making no
-- representation that this implementation is free from any
-- claims of infringement. You are responsible for
-- obtaining any rights you may require for your implementation.
-- Xilinx expressly disclaims any warranty whatsoever with
-- respect to the adequacy of the implementation, including
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied
-- warranties of merchantability or fitness for a particular
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications are
-- expressly prohibited.
--
-- This copyright and support notice must be retained as part
-- of this text at all times. (c) Copyright 2005 Xilinx, Inc.
-- All rights reserved.
--------------------------------------------------------------------------------
--
-- Filename - comp_module.vhd
-- Author - P. Szanto
-- Creation - 27 June 2006
--
-- Description -
--
-- Generic, VHDL comparator
-- Output is "1" when op_a < op_b, output is registered
--
--
-- $RCSfile: comp_module.vhd,v $ $Revision: 1.2 $ $Date: 2006/06/30 19:01:25 $
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity comp_module is
generic(
         DW : integer := 10;
         NUM : integer := 0
);
port(
         clk    : in  std_logic;
         en     : in  std_logic;
         op_a   : in  std_logic_vector(DW downto 0);
         op_b   : in  std_logic_vector(DW downto 0);
         res    : out std_logic_vector(DW downto 0)
);
end comp_module;

architecture rtl of comp_module is

signal sub_res: std_logic_vector(DW downto 0);
attribute syn_dspstyle : string;
attribute syn_dspstyle of sub_res : signal is "logic";
attribute use_dsp48: string;
attribute use_dsp48 of sub_res: signal is "no";

constant CONST_0  : std_logic_vector(127 downto 0) := x"00000000000000000000000000000000";

signal comp_res : std_logic;

begin


process(clk)
begin
if (clk'event and clk='1') then
   if (en='1') then
      sub_res <= op_a - op_b;
   end if;
end if;
end process;

process(clk)
begin
if (clk'event and clk='1') then
   if (en='1') then
      if (op_a < op_b) then   
         comp_res <= '1';
      else
         comp_res <= '0';
      end if;
   end if;
end if;
end process;

res <= comp_res & CONST_0(DW-1 downto 0);

end rtl;
