--------------------------------------------------------------------------------
-- Copyright(C) 2005 by Xilinx, Inc. All rights reserved.
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you
-- a license to use this text/file solely for design, simulation,
-- implementation and creation of design files limited
-- to Xilinx devices or technologies. Use with non-Xilinx
-- devices or technologies is expressly prohibited and
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information
-- "as is" solely for use in developing programs and
-- solutions for Xilinx devices. By providing this design,
-- code, or information as one possible implementation of
-- this feature, application or standard, Xilinx is making no
-- representation that this implementation is free from any
-- claims of infringement. You are responsible for
-- obtaining any rights you may require for your implementation.
-- Xilinx expressly disclaims any warranty whatsoever with
-- respect to the adequacy of the implementation, including
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied
-- warranties of merchantability or fitness for a particular
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications are
-- expressly prohibited.
--
-- This copyright and support notice must be retained as part
-- of this text at all times. (c) Copyright 2005 Xilinx, Inc.
-- All rights reserved.
--------------------------------------------------------------------------------
--
-- Filename - delay_line.vhd
-- Author - P. Szanto
-- Creation - 27 June 2006
--
-- Description -
--
-- Output addressable shift register array
-- Generates SRL_DEPTH deep sub-arrays and uses the output of all such arrays, thus it can be instantly registered
-- Warning: XST 8.1 cannot infer SRLs from the VHDL code, use the instantiated primitives instead
--
--
-- $RCSfile: delay_line.vhd,v $ $Revision: 1.4 $ $Date: 2006/06/30 19:01:23 $
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

use work.rank2d_utils.ALL;

entity delay_line is
generic(SRL_DEPTH : integer := 32;
        SRL_ADDRb : integer := 5;
        NUM_SRL   : integer := 3;
        WIDTH     : integer := 24
      );
port   (clk        : in  std_logic;
        we         : in  std_logic;
        addr       : in  std_logic_vector(SRL_ADDRb-1 downto 0);
        din        : in  std_logic_vector(WIDTH-1 downto 0);
        dout       : out std_logic_vector(WIDTH*NUM_SRL-1 downto 0)
      );
end delay_line;


architecture rtl of delay_line is

type array_SRL_DEPTH_by_WIDTH is array (SRL_DEPTH-1 downto 0) of STD_LOGIC_VECTOR(WIDTH-1 downto 0);
type array_NUMSRL_by_SRL  is array (NUM_SRL-1 downto 0) of array_SRL_DEPTH_by_WIDTH;
signal delay_line : array_NUMSRL_by_SRL;
attribute syn_srlstyle : string;
attribute syn_srlstyle of delay_line : signal is "noextractff_srl"; 
attribute shreg_extract : string;
attribute shreg_extract of delay_line: signal is "yes";

type array_OUT is array (NUM_SRL-1 downto 0) of STD_LOGIC_VECTOR(WIDTH-1 downto 0);
signal out_bus : array_OUT;

Begin

-- generates WIDTH bit wide, NUM_SRL*SRL_DEPTH deep shift register array
-- with one addressable output per SRL

GEN_SRLs:
for ISRL in 0 to NUM_SRL-1 generate
   L0: if (ISRL=0) generate
      process(clk)
      begin
         if (clk'event and clk='1') then
            if (we='1') then
               delay_line(ISRL)(0) <= din;
               for IW in 1 to SRL_DEPTH-1 loop
                  delay_line(ISRL)(IW) <= delay_line(ISRL)(IW-1);
               end loop;
            end if;
         end if;
      end process;
      out_bus(ISRL) <= delay_line(ISRL)(CONV_INTEGER(addr));
   end generate;
   L_OTHERS: if (ISRL/=0) generate
      process(clk)
      begin
         if (clk'event and clk='1') then
            if (we='1') then
               delay_line(ISRL)(0) <= delay_line(ISRL-1)(SRL_DEPTH-1);
               for IW in 1 to SRL_DEPTH-1 loop
                  delay_line(ISRL)(IW) <= delay_line(ISRL)(IW-1);
               end loop;
            end if;
         end if;
      end process;
      out_bus(ISRL) <= delay_line(ISRL)(CONV_INTEGER(addr));
   end generate;
end generate;

GEN_OUT:
for ISRL in 0 to NUM_SRL-1 generate
   dout((ISRL+1)*WIDTH-1 downto ISRL*WIDTH) <= out_bus(ISRL);
end generate;

end rtl;
