--------------------------------------------------------------------------------
-- Copyright(C) 2005 by Xilinx, Inc. All rights reserved.
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you
-- a license to use this text/file solely for design, simulation,
-- implementation and creation of design files limited
-- to Xilinx devices or technologies. Use with non-Xilinx
-- devices or technologies is expressly prohibited and
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information
-- "as is" solely for use in developing programs and
-- solutions for Xilinx devices. By providing this design,
-- code, or information as one possible implementation of
-- this feature, application or standard, Xilinx is making no
-- representation that this implementation is free from any
-- claims of infringement. You are responsible for
-- obtaining any rights you may require for your implementation.
-- Xilinx expressly disclaims any warranty whatsoever with
-- respect to the adequacy of the implementation, including
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied
-- warranties of merchantability or fitness for a particular
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications are
-- expressly prohibited.
--
-- This copyright and support notice must be retained as part
-- of this text at all times. (c) Copyright 2005 Xilinx, Inc.
-- All rights reserved.
--------------------------------------------------------------------------------
--
-- Filename - delay_line_srl32.vhd
-- Author - P. Szanto
-- Creation - 27 June 2006
--
-- Description -
--
-- Output addressable shift register array with instantiated SRL32 primitives for Virtex5
-- Every SRL32 output is used to allow registering them in the rank2d_top module
--
--
-- $RCSfile: delay_line_srl32.vhd,v $ $Revision: 1.2 $ $Date: 2006/06/30 19:01:25 $
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

use work.rank2d_utils.ALL;

library UNISIM;
use UNISIM.VComponents.all;

entity delay_line_srl32 is
generic(SRL_DEPTH : integer := 32;
        SRL_ADDRb : integer := 5;
        NUM_SRL   : integer := 3;
        WIDTH     : integer := 24
      );
port   (clk        : in  std_logic;
        we         : in  std_logic;
        addr       : in  std_logic_vector(SRL_ADDRb-1 downto 0);
        din        : in  std_logic_vector(WIDTH-1 downto 0);
        dout       : out std_logic_vector(WIDTH*NUM_SRL-1 downto 0)
      );
end delay_line_srl32;


architecture struct of delay_line_srl32 is

component SRLC32E
generic(
   INIT : std_logic_vector(15 downto 0));
port(
   Q   : out std_logic;
   Q31 : out std_logic;
   A0  : in  std_logic;
   A1  : in  std_logic;
   A2  : in  std_logic;
   A3  : in  std_logic;
   A4  : in  std_logic;
   CE  : in  std_logic;
   CLK : in  std_logic;
   D   : in  std_logic
);
end component;

type array_NUMSRL_by_WIDTH is array (NUM_SRL-1 downto 0) of std_logic_vector(WIDTH-1 downto 0);
signal srl_out_cw : array_NUMSRL_by_WIDTH;
signal srl_out_dw : array_NUMSRL_by_WIDTH;

Begin

-- generates WIDTH bit wide, NUM_SRL*SRL_DEPTH deep shift register array
-- with one addressable output per SRL

GEN_SRLs:
for ISRL in 0 to NUM_SRL-1 generate
   L0: 
   if (ISRL=0) generate
      GEN_SRL32s:
      for IW in 0 to WIDTH-1 generate
         SRLC32E_i : SRLC32E
         generic map (
            INIT => X"0000")
         port map (
            Q   => srl_out_dw(ISRL)(IW),
            Q31 => srl_out_cw(ISRL)(IW),
            A0  => addr(0),
            A1  => addr(1),
            A2  => addr(2),
            A3  => addr(3),
            A4  => addr(4),
            CE  => we,
            CLK => clk,
            D   => din(IW)
         );
      end generate;
   end generate;
   L_OTHERS: 
   if (ISRL/=0) generate
      GEN_SRL32s:
      for IW in 0 to WIDTH-1 generate
         SRLC32E_i : SRLC32E
         generic map (
            INIT => X"0000")
         port map (
            Q   => srl_out_dw(ISRL)(IW),
            Q31 => srl_out_cw(ISRL)(IW),
            A0  => addr(0),
            A1  => addr(1),
            A2  => addr(2),
            A3  => addr(3),
            A4  => addr(4),
            CE  => we,
            CLK => clk,
            D   => srl_out_cw(ISRL-1)(IW)
         );
      end generate;
   end generate;
end generate;

GEN_OUT:
for ISRL in 0 to NUM_SRL-1 generate
   dout((ISRL+1)*WIDTH-1 downto ISRL*WIDTH) <= srl_out_dw(ISRL);
end generate;

end struct;
