--------------------------------------------------------------------------------
-- Copyright(C) 2005 by Xilinx, Inc. All rights reserved.
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you
-- a license to use this text/file solely for design, simulation,
-- implementation and creation of design files limited
-- to Xilinx devices or technologies. Use with non-Xilinx
-- devices or technologies is expressly prohibited and
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information
-- "as is" solely for use in developing programs and
-- solutions for Xilinx devices. By providing this design,
-- code, or information as one possible implementation of
-- this feature, application or standard, Xilinx is making no
-- representation that this implementation is free from any
-- claims of infringement. You are responsible for
-- obtaining any rights you may require for your implementation.
-- Xilinx expressly disclaims any warranty whatsoever with
-- respect to the adequacy of the implementation, including
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied
-- warranties of merchantability or fitness for a particular
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications are
-- expressly prohibited.
--
-- This copyright and support notice must be retained as part
-- of this text at all times. (c) Copyright 2005 Xilinx, Inc.
-- All rights reserved.
--------------------------------------------------------------------------------
--
-- Filename - fvg_sum.vhd
-- Author - P. Szanto
-- Creation - 27 June 2006
--
-- Description -
--
-- Channel summation FVG module
-- Sums the three input channels in two pipeline stages
-- If the output data width is smaller than required two represent the full result,
-- the significant bit are used.
-- If parameters are not set correctly, result is extended with "0"
--
--
-- $RCSfile: fvg_sum.vhd,v $ $Revision: 1.5 $ $Date: 2006/06/30 19:01:23 $
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

use work.rank2d_utils.ALL;

entity fvg_sum is
generic(
        DW_CC0     : integer := 8;
        DW_CC1     : integer := 8;
        DW_CC2     : integer := 8;
        DW         : integer := 10
      );
port   (
        clk        : in  std_logic;
        en         : in  std_logic;
        cc0_in     : in  std_logic_vector(DW_CC0-1 downto 0);
        cc1_in     : in  std_logic_vector(DW_CC1-1 downto 0);
        cc2_in     : in  std_logic_vector(DW_CC2-1 downto 0);
        filter_val : out std_logic_vector(DW-1 downto 0)
      );
end fvg_sum;


architecture rtl of fvg_sum is

constant DW_SUM0 : integer := MAX(DW_CC0+1, DW_CC1+1);
constant DW_SUM1 : integer := MAX(DW_SUM0+1, DW_CC2+1);

constant CONST_0  : std_logic_vector(127 downto 0) := x"00000000000000000000000000000000";

signal sum0_op0  : std_logic_vector(DW_SUM0-1  downto 0);
signal sum0_op1  : std_logic_vector(DW_SUM0-1  downto 0);
signal sum_cc01  : std_logic_vector(DW_SUM0-1  downto 0);
signal cc2_dl    : std_logic_vector(DW_CC2-1 downto 0);

signal sum1_op0  : std_logic_vector(DW_SUM1-1 downto 0);
signal sum1_op1  : std_logic_vector(DW_SUM1-1 downto 0);
signal sum_cc012 : std_logic_vector(DW_SUM1-1 downto 0);

begin


sum0_op0 <= CONST_0(DW_SUM0-DW_CC0-1 downto 0) & cc0_in;
sum0_op1 <= CONST_0(DW_SUM0-DW_CC1-1 downto 0) & cc1_in;

sum1_op0 <= CONST_0(DW_SUM1-1 downto DW_SUM0) & sum_cc01;
sum1_op1 <= CONST_0(DW_SUM1-1 downto DW_CC2) & cc2_dl;

process(clk)
begin
if (clk'event and clk='1') then
   if (en='1') then
      sum_cc01 <= sum0_op0 + sum0_op1;
      cc2_dl <= cc2_in;
   end if;
end if;
end process;

process(clk)
begin
if (clk'event and clk='1') then
   if (en='1') then
      sum_cc012 <= sum1_op0 + sum1_op1;
   end if;
end if;
end process;

GEN_OUT_LE:
if (DW <= DW_SUM1) generate
   filter_val <= sum_cc012(DW_SUM1-1 downto DW_SUM1-DW);
end generate;
GEN_OUT_GR:
if (DW > DW_SUM1) generate
   filter_val <= CONST_0(DW-DW_SUM1-1 downto 0) & sum_cc012;
end generate;

end rtl;
