--------------------------------------------------------------------------------
-- Copyright(C) 2005 by Xilinx, Inc. All rights reserved.
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you
-- a license to use this text/file solely for design, simulation,
-- implementation and creation of design files limited
-- to Xilinx devices or technologies. Use with non-Xilinx
-- devices or technologies is expressly prohibited and
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information
-- "as is" solely for use in developing programs and
-- solutions for Xilinx devices. By providing this design,
-- code, or information as one possible implementation of
-- this feature, application or standard, Xilinx is making no
-- representation that this implementation is free from any
-- claims of infringement. You are responsible for
-- obtaining any rights you may require for your implementation.
-- Xilinx expressly disclaims any warranty whatsoever with
-- respect to the adequacy of the implementation, including
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied
-- warranties of merchantability or fitness for a particular
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications are
-- expressly prohibited.
--
-- This copyright and support notice must be retained as part
-- of this text at all times. (c) Copyright 2005 Xilinx, Inc.
-- All rights reserved.
--------------------------------------------------------------------------------
--
-- Filename - rank2d_latency.vhd
-- Author - P. Szanto
-- Creation - 27 June 2006
--
-- Description -
--
-- Support package for the rank filter to compute 
--    configuration dependent parameters (latency of the diferent modules)
--    family dependent parameters (SRL depth, fanout limits,..)
--
--
-- $RCSfile: rank2d_latency.vhd,v $ $Revision: 1.2 $ $Date: 2006/06/30 19:01:26 $
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;

use work.rank2d_utils.ALL;

package rank2d_latency is

-- Declare functions and procedure
function GET_1CNT_WIDTH (F2_TYPE: integer; TAP: integer; TAP_WM: integer; CYCLES_PER_COL: integer) return integer;
function GET_1CNT_STAGES (TAP: integer; ADDER_INS: integer) return integer;
function GET_1CNT_ADDERS (STAGE1: integer; STAGE: integer) return integer;
function GET_FVG_LATENCY (FVG_TYPE: integer) return integer;
function GET_BRAM_DELAY(FAMILY: string) return integer;

function GET_ADDER_INS(FAMILY: string) return integer;
function GET_SRL_DEPTH(FAMILY: string) return integer;
function GET_BRAM_IN_FAN (FAMILY: string; TAP: integer) return integer;
function GET_FAMILY_HAS_MAC(FAMILY: string) return integer;
function GET_FABRIC_ADDS(FAMILY: string) return integer;

end rank2d_latency;


package body rank2d_latency is


-- ------------------------------------------------------------------------ 

function GET_1CNT_WIDTH (F2_TYPE: integer; TAP: integer; TAP_WM: integer; CYCLES_PER_COL: integer) return integer is
begin 
   if (F2_TYPE=0) then
      return TAP;
   else
      return TAP_WM;--DIV_CEIL(TAP_WM, CYCLES_PER_COL);
   end if;
end GET_1CNT_WIDTH;

-- ------------------------------------------------------------------------ 

function GET_1CNT_STAGES (TAP: integer; ADDER_INS: integer) return integer is
variable cnt1_1stage : integer;
variable cnt1_stages : integer;
begin 
   cnt1_1stage := MAX(1, DIV_CEIL_2PWR(TAP, ADDER_INS));
   cnt1_stages := MAX(1, LOG2_CEIL(cnt1_1stage+1));
   return cnt1_stages;
end GET_1CNT_STAGES;

-- ------------------------------------------------------------------------ 
function GET_FVG_LATENCY (FVG_TYPE: integer) return integer is
begin
   if (FVG_TYPE=0) then
      return 2;
   elsif (FVG_TYPE=1) then
      return 4;
   elsif (FVG_TYPE=2) then
      return 0;
   else
      return 0;
   end if;
end GET_FVG_LATENCY;
-- ------------------------------------------------------------------------    
function GET_1CNT_ADDERS (STAGE1: integer; STAGE: integer) return integer is
variable IST : integer;
variable OUTS : integer;
variable ADDERS : integer;
begin
   IST  := 0;
   OUTS := STAGE1;
   while (IST < STAGE) loop
      ADDERS := OUTS/2;
      OUTS := OUTS/2 + (OUTS - 2*(OUTS/2));
      IST := IST + 1;
   end loop;
   return ADDERS;
end GET_1CNT_ADDERS;
-- ------------------------------------------------------------------------    
function GET_BRAM_DELAY(FAMILY: string) return integer is
begin
   if (FAMILY="virtex4" or FAMILY="virtex5") then
      return 4;
   else
      return 3;
   end if;
end GET_BRAM_DELAY;
-- ------------------------------------------------------------------------    
-- ------------------------------------------------------------------------    
function GET_ADDER_INS(FAMILY: string) return integer is
begin
   if (FAMILY="virtex5") then
      return 6;
   else
      return 3;
   end if;
end GET_ADDER_INS;
-- ------------------------------------------------------------------------    
function GET_SRL_DEPTH(FAMILY: string) return integer is
begin
   if (FAMILY="virtex5") then
      return 32;
   else
      return 16;
   end if;
end GET_SRL_DEPTH;
-- ------------------------------------------------------------------------    
function GET_BRAM_IN_FAN (FAMILY: string; TAP: integer) return integer is
begin
   if (FAMILY="virtex4" or FAMILY="virtex5") then
      return 10000;
   else
      if (TAP <= 9) then
         return 10000;
      else
         return 1;
      end if;
   end if;
end GET_BRAM_IN_FAN;
-- ------------------------------------------------------------------------    
function GET_FAMILY_HAS_MAC(FAMILY: string) return integer is
begin
   if (FAMILY="virtex4" or FAMILY="virtex5") then
      return 1;
   else
      return 0;
   end if;
end GET_FAMILY_HAS_MAC;
-- ------------------------------------------------------------------------    
function GET_FABRIC_ADDS(FAMILY: string) return integer is
begin
   if (FAMILY="virtex4" or FAMILY="virtex5") then
      return 0;
   else
      return 1;
   end if;
end GET_FABRIC_ADDS;
   
end rank2d_latency;
