--------------------------------------------------------------------------------
-- Copyright(C) 2005 by Xilinx, Inc. All rights reserved.
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you
-- a license to use this text/file solely for design, simulation,
-- implementation and creation of design files limited
-- to Xilinx devices or technologies. Use with non-Xilinx
-- devices or technologies is expressly prohibited and
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information
-- "as is" solely for use in developing programs and
-- solutions for Xilinx devices. By providing this design,
-- code, or information as one possible implementation of
-- this feature, application or standard, Xilinx is making no
-- representation that this implementation is free from any
-- claims of infringement. You are responsible for
-- obtaining any rights you may require for your implementation.
-- Xilinx expressly disclaims any warranty whatsoever with
-- respect to the adequacy of the implementation, including
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied
-- warranties of merchantability or fitness for a particular
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications are
-- expressly prohibited.
--
-- This copyright and support notice must be retained as part
-- of this text at all times. (c) Copyright 2005 Xilinx, Inc.
-- All rights reserved.
--------------------------------------------------------------------------------
--
-- Filename - rank2d_utils.vhd
-- Author - P. Szanto
-- Creation - 27 June 2006
--
-- Description -
--
-- Arithmetic support package for the rank filter to compute parameters
--
-- $RCSfile: rank2d_utils.vhd,v $ $Revision: 1.2 $ $Date: 2006/06/30 19:01:26 $
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.all;

package rank2d_utils is

-- Declare functions and procedure
  function MIN (a: integer; b: integer) return integer;
  function MAX (a: integer; b: integer) return integer;

  function LOG2_BASE  (a: integer) return integer;
  function LOG2_CEIL  (a: integer) return integer;
  function DIV_CEIL   (a: integer; b: integer) return integer;
  function DIV_CEIL_2PWR (a: integer; b: integer) return integer;

  function GET_TAPV (TAPr: integer; WV_real: integer; NI: integer) return integer;
  function GET_TAPb (TAPr: integer; WV_real: integer; NI: integer) return integer;

  function str(int: integer; base: integer) return string;
  function chr(int: integer) return character;

end rank2d_utils;


package body rank2d_utils is


-- ------------------------------------------------------------------------ 



  function MIN( a: integer; b: integer)	return integer is

  begin 

    if (a<b) then return a; else return b; end if; 

  end MIN;

  

-- ------------------------------------------------------------------------ 

  function MAX( a: integer; b: integer)	return integer is
  begin 
    if (a>b) then return a; else return b; end if; 
  end MAX;

  
-- ------------------------------------------------------------------------ 

  FUNCTION LOG2_BASE(a:INTEGER) RETURN INTEGER IS
    VARIABLE INDEX 	: INTEGER := a;
    VARIABLE SUM    : INTEGER := 1; 
	BEGIN
		CASE INDEX IS
			WHEN 0 to 1 => SUM := 0;
			WHEN 2 to 3 => SUM := 1;
			WHEN 4 to 7 => SUM := 2;
			WHEN 8 to 15 => SUM := 3;
			WHEN 16 to 31 => SUM := 4;
			WHEN 33 to 63 => SUM := 5;
			WHEN 64 to 127 => SUM := 6;
			WHEN 128 to 255 => SUM := 7;   
			WHEN 256 to 511 => SUM := 8;
			WHEN 512 to 1023 => SUM := 9;
			WHEN 1024 to 2047 => SUM := 10;
			WHEN 2048 to 4095 => SUM := 11;
			WHEN 4096 to 8191 => SUM := 12;
			WHEN 8192 to 16383 => SUM := 13;
			WHEN 16384 to 32785 => SUM := 14;
			WHEN 32796 to 65535	=> SUM := 15;
			WHEN 65536 to 131071 => SUM := 16;
			WHEN 131072 to 262143 => SUM := 17;
			WHEN 262144 to 524287 => SUM := 18;
			WHEN 524288 to 1048575 => SUM := 19;
			WHEN 1048576 to 2097151 => SUM := 20;
			WHEN 2097152 to 4194303 => SUM := 21;
			WHEN 4194304 to 8388607 => SUM := 22;
			WHEN 8388608 to 16777215 => SUM := 23;
			WHEN OTHERS => SUM := 24;
			END CASE; 
		RETURN SUM;
    END LOG2_BASE; 

-- ------------------------------------------------------------------------ 

  FUNCTION LOG2_CEIL(a:INTEGER) RETURN INTEGER IS
    VARIABLE INDEX 	: INTEGER := a;
    VARIABLE SUM    : INTEGER := 1; 
	BEGIN
		CASE INDEX IS
			WHEN 0 to 2 => SUM := 1;
			WHEN 3 to 4 => SUM := 2;
			WHEN 5 to 8 => SUM := 3;
			WHEN 9 to 16 => SUM := 4;
			WHEN 17 to 32 => SUM := 5;
			WHEN 33 to 64 => SUM := 6;
			WHEN 65 to 128 => SUM := 7;
			WHEN 129 to 256 => SUM := 8;   
			WHEN 257 to 512 => SUM := 9;
			WHEN 513 to 1024 => SUM := 10;
			WHEN 1025 to 2048 => SUM := 11;
			WHEN 2049 to 4096 => SUM := 12;
			WHEN 4097 to 8192 => SUM := 13;
			WHEN 8193 to 16384 => SUM := 14;
			WHEN 16385 to 32786 => SUM := 15;
			WHEN 32797 to 65536	=> SUM := 16;
			WHEN 65537 to 131072 => SUM := 17;
			WHEN 131073 to 262144 => SUM := 18;
			WHEN 262145 to 524288 => SUM := 19;
			WHEN 524289 to 1048576 => SUM := 20;
			WHEN 1048577 to 2097152 => SUM := 21;
			WHEN 2097153 to 4194304 => SUM := 22;
			WHEN 4194305 to 8388608 => SUM := 23;
			WHEN 8388609 to 16777216 => SUM := 24;
			WHEN OTHERS => SUM := 25;
			END CASE; 
		RETURN SUM;
    END LOG2_CEIL; 

-- ------------------------------------------------------------------------ 

  FUNCTION DIV_CEIL(a:INTEGER; b: INTEGER)
	RETURN INTEGER IS
    VARIABLE div_res	: INTEGER;
    VARIABLE div_mod : INTEGER; 
	 VARIABLE res     : INTEGER; 
	BEGIN
		div_res := a/b;
		div_mod := a mod b;
		if (div_mod = 0) then
			res := div_res;
		else
			res := div_res + 1;
		end if;
		RETURN res;
   END DIV_CEIL; 


-- ------------------------------------------------------------------------ 

  FUNCTION DIV_CEIL_2PWR(a:INTEGER; b: INTEGER)
	RETURN INTEGER IS
    VARIABLE div_res	: INTEGER;
    VARIABLE div_mod : INTEGER; 
	 VARIABLE res     : INTEGER; 
	BEGIN
		div_res := a/b;
		div_mod := a mod b;
		if (div_mod = 0) then
			res := div_res;
		else
			res := div_res + 1;
		end if;
      while (2**LOG2_BASE(res) /= res) loop
         res := res + 1;
      end loop;
		RETURN res;
   END DIV_CEIL_2PWR; 

-- ------------------------------------------------------------------------ 
   
  function GET_TAPV (TAPr: integer; WV_real: integer; NI: integer)
   return integer is
   variable WH : integer;
   variable WV_virt : integer;
   variable TAP : integer;
  begin
   WH      := TAPr/WV_real;
   WV_virt := DIV_CEIL(WV_real, NI)*NI;
   TAP     := WV_virt*WH; 
   return TAP;
  end GET_TAPV;

-- ------------------------------------------------------------------------ 
   
  function GET_TAPb (TAPr: integer; WV_real: integer; NI: integer)
   return integer is
   variable WH : integer;
   variable WV_virt : integer;
   variable TAP : integer;
   variable TAPb : integer;
  begin
   WH      := TAPr/WV_real;
   WV_virt := DIV_CEIL(WV_real, NI)*NI;
   TAP     := WV_virt*WH; 
   TAPb    := LOG2_CEIL(TAP);
   return TAPb;
  end GET_TAPb;

-- ------------------------------------------------------------------------

   function str(int: integer; base: integer) return string is

    variable temp:      string(1 to 10);
    variable num:       integer;
    variable abs_int:   integer;
    variable len:       integer := 1;
    variable power:     integer := 1;

   begin

    -- bug fix for negative numbers
    abs_int := abs(int);

    num     := abs_int;

    while num >= base loop                     -- Determine how many
      len := len + 1;                          -- characters required
      num := num / base;                       -- to represent the
    end loop ;                                 -- number.

    for i in len downto 1 loop                 -- Convert the number to
      temp(i) := chr(abs_int/power mod base);  -- a string starting
      power := power * base;                   -- with the right hand
    end loop ;                                 -- side.

    -- return result and add sign if required
    if int < 0 then
       return '-'& temp(1 to len);
     else
       return temp(1 to len);
    end if;

   end str;  

-- ------------------------------------------------------------------------

   function chr(int: integer) return character is
    variable c: character;
   begin
        case int is
          when  0 => c := '0';
          when  1 => c := '1';
          when  2 => c := '2';
          when  3 => c := '3';
          when  4 => c := '4';
          when  5 => c := '5';
          when  6 => c := '6';
          when  7 => c := '7';
          when  8 => c := '8';
          when  9 => c := '9';
          when 10 => c := 'A';
          when 11 => c := 'B';
          when 12 => c := 'C';
          when 13 => c := 'D';
          when 14 => c := 'E';
          when 15 => c := 'F';
          when 16 => c := 'G';
          when 17 => c := 'H';
          when 18 => c := 'I';
          when 19 => c := 'J';
          when 20 => c := 'K';
          when 21 => c := 'L';
          when 22 => c := 'M';
          when 23 => c := 'N';
          when 24 => c := 'O';
          when 25 => c := 'P';
          when 26 => c := 'Q';
          when 27 => c := 'R';
          when 28 => c := 'S';
          when 29 => c := 'T';
          when 30 => c := 'U';
          when 31 => c := 'V';
          when 32 => c := 'W';
          when 33 => c := 'X';
          when 34 => c := 'Y';
          when 35 => c := 'Z';
          when others => c := '?';
        end case;
        return c;
    end chr;   
   
   
end rank2d_utils;
