--------------------------------------------------------------------------------
-- Copyright(C) 2005 by Xilinx, Inc. All rights reserved.
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you
-- a license to use this text/file solely for design, simulation,
-- implementation and creation of design files limited
-- to Xilinx devices or technologies. Use with non-Xilinx
-- devices or technologies is expressly prohibited and
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information
-- "as is" solely for use in developing programs and
-- solutions for Xilinx devices. By providing this design,
-- code, or information as one possible implementation of
-- this feature, application or standard, Xilinx is making no
-- representation that this implementation is free from any
-- claims of infringement. You are responsible for
-- obtaining any rights you may require for your implementation.
-- Xilinx expressly disclaims any warranty whatsoever with
-- respect to the adequacy of the implementation, including
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied
-- warranties of merchantability or fitness for a particular
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications are
-- expressly prohibited.
--
-- This copyright and support notice must be retained as part
-- of this text at all times. (c) Copyright 2005 Xilinx, Inc.
-- All rights reserved.
--------------------------------------------------------------------------------
--
-- Filename - sub_inst.vhd
-- Author - P. Szanto
-- Creation - 27 June 2006
--
-- Description -
-- Comparator module with instantiated primitives for families other than Virtex2, Virtex2Pro, Virtex4 and Virtex5
-- Implemented as a subtractor, the result is the last carry out, which is registered
-- The last primitives are RLOCed to force PAR to place them within the same CLB
-- 
-- USE ONLY when you have timing problems and par does not place the last MUXCY, XORCY and the output FF
-- into the same CLB
--
--
-- $RCSfile: sub_inst.vhd,v $ $Revision: 1.0 $ $Date: 2006/02/09 01:10:49 $
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

Library UNISIM;
use UNISIM.vcomponents.all;

library work;
use work.rank2d_utils.ALL;

entity sub_inst is
generic(
         DW : integer := 10;
         NUM : integer := 0
);
port(
         clk    : in  std_logic;
         en     : in  std_logic;
         op_a   : in  std_logic_vector(DW downto 0);
         op_b   : in  std_logic_vector(DW downto 0);
         res    : out std_logic_vector(DW downto 0)
);
end sub_inst;

architecture struct of sub_inst is

component MUXCY_L
port (
   LO : out std_logic;
   CI : in  std_logic;
   DI : in  std_logic;
   S  : in  std_logic
);
end component;

component XORCY_L
port (
   LO : out std_logic;
   CI : in  std_logic;
   LI : in  std_logic
);
end component;

component FDCE
generic (
   INIT : bit);
port (
   Q   : out std_logic;
   C   : in  std_logic;
   CE  : in  std_logic;
   CLR : in  std_logic;
   D   : in  std_logic
);
end component;

signal muxcy_out : std_logic_vector(DW downto 0);
signal mux_sel   : std_logic_vector(DW downto 0);

signal sub_res : std_logic_vector(DW downto 0);

attribute uset : string;
attribute uset of XORCY_L_DW: label is str(NUM, 10);
attribute uset of reg_out_dw: label is str(NUM, 10);

attribute xc_uset : string;
attribute xc_uset of XORCY_L_DW: label is str(NUM, 10);
attribute xc_uset of reg_out_dw: label is str(NUM, 10);

attribute rloc: string;
attribute rloc of XORCY_L_DW: label  is "X0Y0";
attribute rloc of reg_out_dw: label is "X0Y0";

begin

GEN_MUXCY:
for I in 0 to DW-1 generate
   OP0:
   if (I=0) generate
      mux_sel(I) <= (not op_a(I)) xor op_b(I);
      MUXCY_L_i : MUXCY_L
      port map (
         LO => muxcy_out(I),
         CI => '1',
         DI => op_a(I),
         S  => mux_sel(I)
      );
   end generate;
   OP_OHER:
   if (I /= 0) generate
      mux_sel(I) <= (not op_a(I)) xor op_b(I);
      MUXCY_L_i : MUXCY_L
      port map (
         LO => muxcy_out(I),
         CI => muxcy_out(I-1),
         DI => op_a(I),
         S  => mux_sel(I)
      );   
   end generate;
end generate;


mux_sel(DW) <= (not op_a(DW)) xor op_b(DW);
MUXCY_L_DW : MUXCY_L
port map (
   LO => muxcy_out(DW),
   CI => muxcy_out(DW-1),
   DI => op_a(DW),
   S  => mux_sel(DW)
);   

XORCY_L_DW : XORCY_L
port map (
   LO => sub_res(DW), -- XOR local output signal
   CI => muxcy_out(DW), -- Carry input signal
   LI => mux_sel(DW)  -- LUT4 input signal
);   

reg_out_dw: FDCE
generic map (
   INIT => '0')
port map (
   Q => res(DW),
   C => clk,
   CE => en,
   CLR => '0',
   D => sub_res(DW)
);

res(DW-1 downto 0) <= "0000000000";

end struct;

