--------------------------------------------------------------------------------
-- Copyright(C) 2005 by Xilinx, Inc. All rights reserved.
-- This text/file contains proprietary, confidential
-- information of Xilinx, Inc., is distributed under license
-- from Xilinx, Inc., and may be used, copied and/or
-- disclosed only pursuant to the terms of a valid license
-- agreement with Xilinx, Inc. Xilinx hereby grants you
-- a license to use this text/file solely for design, simulation,
-- implementation and creation of design files limited
-- to Xilinx devices or technologies. Use with non-Xilinx
-- devices or technologies is expressly prohibited and
-- immediately terminates your license unless covered by
-- a separate agreement.
--
-- Xilinx is providing this design, code, or information
-- "as is" solely for use in developing programs and
-- solutions for Xilinx devices. By providing this design,
-- code, or information as one possible implementation of
-- this feature, application or standard, Xilinx is making no
-- representation that this implementation is free from any
-- claims of infringement. You are responsible for
-- obtaining any rights you may require for your implementation.
-- Xilinx expressly disclaims any warranty whatsoever with
-- respect to the adequacy of the implementation, including
-- but not limited to any warranties or representations that this
-- implementation is free from claims of infringement, implied
-- warranties of merchantability or fitness for a particular
-- purpose.
--
-- Xilinx products are not intended for use in life support
-- appliances, devices, or systems. Use in such applications are
-- expressly prohibited.
--
-- This copyright and support notice must be retained as part
-- of this text at all times. (c) Copyright 2005 Xilinx, Inc.
-- All rights reserved.
--------------------------------------------------------------------------------
--
-- Filename - sub_inst_v2.vhd
-- Author - P. Szanto
-- Creation - 27 June 2006
--
-- Description -
-- Comparator module with instantiated primitives for Virtex2 family
-- Implemented as a subtractor, extended to have even input width
-- This way the ORCY above the top LUT can be used
-- The last MUXCY, ORCY and output FF are RLOCed to be placed within the same CLB
-- 
-- USE ONLY when you have timing problems and par does not place the last MUXCY, ORCY and the output FF
-- into the same CLB
--
--
-- $RCSfile: sub_inst_v2.vhd,v $ $Revision: 1.0 $ $Date: 2006/02/09 01:10:49 $
-- 
--------------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

library work;
use work.rank2d_utils.ALL;

Library UNISIM;
use UNISIM.vcomponents.all;

library work;
use work.all;

entity sub_inst_v2 is
generic(
         DW : integer := 10;
         NUM : integer := 0
);
port(
         clk    : in  std_logic;
         en     : in  std_logic;
         op_a   : in  std_logic_vector(DW downto 0);
         op_b   : in  std_logic_vector(DW downto 0);
         res    : out std_logic_vector(DW downto 0)
);
end sub_inst_v2;

architecture struct of sub_inst_v2 is

constant cy_levels : integer := DIV_CEIL(DW, 2)*2;

signal muxcy_out : std_logic_vector(cy_levels-1 downto 0);
signal mux_sel   : std_logic_vector(cy_levels-1 downto 0);

signal sub_res : std_logic_vector(cy_levels-1 downto 0);

attribute uset : string;
attribute uset of MUXCY_L_DW: label is str(NUM, 10);
attribute uset of ORCY_L_DW: label is str(NUM, 10);
attribute uset of reg_out_dw: label is str(NUM, 10);

attribute xc_uset : string;
attribute xc_uset of MUXCY_L_DW: label is str(NUM, 10);
attribute xc_uset of ORCY_L_DW: label is str(NUM, 10);
attribute xc_uset of reg_out_dw: label is str(NUM, 10);

attribute rloc: string;
attribute rloc of MUXCY_L_DW: label  is "X0Y0";
attribute rloc of ORCY_L_DW: label  is "X0Y0";
attribute rloc of reg_out_dw: label is "X0Y0";

begin

GEN_MUXCY:
for I in 0 to cy_levels-2 generate
   OP0:
   if (I=0) generate
      mux_sel(I) <= (not op_a(I)) xor op_b(I);
      MUXCY_L_i : MUXCY_L
      port map (
         LO => muxcy_out(I),
         CI => '1',
         DI => op_a(I),
         S  => mux_sel(I)
      );
   end generate;
   OP_OHER:
   if (I /= 0) generate
      mux_sel(I) <= (not op_a(I)) xor op_b(I);
      MUXCY_L_i : MUXCY_L
      port map (
         LO => muxcy_out(I),
         CI => muxcy_out(I-1),
         DI => op_a(I),
         S  => mux_sel(I)
      );   
   end generate;
end generate;


mux_sel(cy_levels-1) <= (not op_a(cy_levels-1)) xor op_b(cy_levels-1);
MUXCY_L_DW : MUXCY
port map (
   O  => muxcy_out(cy_levels-1),
   CI => muxcy_out(cy_levels-2),
   DI => op_a(cy_levels-1),
   S  => mux_sel(cy_levels-1)
);   

ORCY_L_DW : ORCY
port map (
   O  => sub_res(cy_levels-1),
   CI => muxcy_out(cy_levels-1),
   I  => '0'
);   

reg_out_dw: FDCE
generic map (
   INIT => '0')
port map (
   Q => res(DW),
   C => clk,
   CE => en,
   CLR => '0',
   D => sub_res(cy_levels-1)
);

res(DW-1 downto 0) <= "0000000000";

end struct;
