function [data_out] = rank1D(data_in, NTAPs, RANK)
% MATLAB model for the one dimensional rank-order filter

  if (RANK>NTAPs) error('Parameter RANK should be less or equal than NTAPs') 
  end;

  % Transform input data array to row vector if needed:
  len = product(size(data_in));
  data = reshape(data_in, 1, len);

  % Initial conditions:
  D  = zeros(1,NTAPs); 
  SH = zeros(NTAPs-1,NTAPs); % 0:1:(NTAPs-1);
  data_out = [];

  for i=1:len
    D = [ D(2:NTAPs), data(i)]
    comp  = (D(1:NTAPs-1) >= data(i))
    SH    = [ [SH(2:NTAPs-1,2:NTAPs); comp], (1-comp)' ]
    cnt1s = sum(SH)
    addr  = find(cnt1s == RANK)
    if (i>NTAPs-1)
      if (isempty(addr))  data_out = [data_out, NaN]; 
      else                data_out = [data_out, D(addr)]; 
      end;
    end;
  end;
end

