function output_image = rank2d_golden( input_image, WH, WW, rank, Y_TYPE )
  % Golden Matlab reference for the 2D rank filter
  % output image is WH rows, WW cols smaller than input_image
  % Padd input image to ensure similar input-output sizes
  
  [size_y, size_x, planes] = size(input_image);
  % make sure output_image is the same type as input_image:
  output_image = input_image( 1:1+size_y-WH, 1:1+size_x-WW, 1:planes ); 
  TAPs=WH*WW;
  coeffs = zeros(planes, 1); coeffs(1:3) = [0.50989; 1; 0.19421]; % Y generator coeffs
  
  for row=1:1+size_y-WH
    for col=1:1+size_x-WW
      filter_window = reshape( input_image(row:row+WH-1, col:col+WW-1, :), TAPs, planes);
      switch (Y_TYPE)
        case 0    
          y_window = sum(filter_window')';
        case 1    
          y_window = double(filter_window)*coeffs;
        otherwise
          y_window = filter_window(:,1);
      end
      [ranked,index] = sort(y_window);
      output_image(row, col,:) = filter_window(index(rank),:);
    end
  end



