library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--library work;
use work.rank2d_utils.ALL;
use work.rank2d_latency.ALL;

entity test_top is
generic(DATA_WIDTH_CH0     : integer := 8;      -- Bit width of color channel 0
        DATA_WIDTH_CH1     : integer := 8;      -- Bit width of color channel 1
        DATA_WIDTH_CH2     : integer := 8;      -- Bit width of color channel 2
        DATA_WIDTH_FILTER  : integer := 10;     -- Filter data bit width
        WINDOW_WIDTH       : integer := 3;      -- Horizontal size of filter window
        WINDOW_HEIGHT      : integer := 3;      -- Vertical size of filter window
        NEW_INPUTS         : integer := 1;      -- new input samples/clk for the filter core
        MAX_HORIZONTAL_RES : integer := 2048;   -- maximum number of pixels per line
        Y_GENERATOR_TYPE   : integer := 2;      -- Filter Value Generator Type
        FAMILY             : string  := "spartan3");
port   (clk       : in  std_logic;
        rst       : in  std_logic;
        ce        : in  std_logic;
        pix_en_in : in  std_logic;
        rank      : in  std_logic_vector(GET_TAPb (WINDOW_HEIGHT*WINDOW_WIDTH,WINDOW_HEIGHT,NEW_INPUTS)-1 downto 0);
        cc0_in    : in  std_logic_vector(DATA_WIDTH_CH0-1 downto 0);
        cc1_in    : in  std_logic_vector(DATA_WIDTH_CH1-1 downto 0);
        cc2_in    : in  std_logic_vector(DATA_WIDTH_CH2-1 downto 0);
        vs_in     : in  std_logic;
        hs_in     : in  std_logic;
        cc0_out   : out std_logic_vector(DATA_WIDTH_CH0-1 downto 0);
        cc1_out   : out std_logic_vector(DATA_WIDTH_CH1-1 downto 0);
        cc2_out   : out std_logic_vector(DATA_WIDTH_CH2-1 downto 0);
        pix_en_out: out std_logic;
        vs_out    : out std_logic;
        hs_out    : out std_logic
);
end test_top;

architecture rtl of test_top is

signal ce_r        : std_logic;
signal pix_en_in_r : std_logic;
signal rank_r      : std_logic_vector(GET_TAPb (WINDOW_HEIGHT*WINDOW_WIDTH,WINDOW_HEIGHT,NEW_INPUTS)-1 downto 0);
signal cc0_in_r    : std_logic_vector(DATA_WIDTH_CH0-1 downto 0);
signal cc1_in_r    : std_logic_vector(DATA_WIDTH_CH1-1 downto 0);
signal cc2_in_r    : std_logic_vector(DATA_WIDTH_CH2-1 downto 0);
signal vs_in_r     : std_logic;
signal hs_in_r     : std_logic;
signal cc0_out_w   : std_logic_vector(DATA_WIDTH_CH0-1 downto 0);
signal cc1_out_w   : std_logic_vector(DATA_WIDTH_CH1-1 downto 0);
signal cc2_out_w   : std_logic_vector(DATA_WIDTH_CH2-1 downto 0);
signal pix_en_out_w: std_logic;
signal vs_out_w    : std_logic;
signal hs_out_w    : std_logic;

Begin

process(clk)
begin
if (clk'event and clk='1') then
   ce_r        <= ce;
   pix_en_in_r <= pix_en_in;
   rank_r      <= rank;
   cc0_in_r    <= cc0_in;
   cc1_in_r    <= cc1_in;
   cc2_in_r    <= cc2_in;
   vs_in_r     <= vs_in;
   hs_in_r     <= hs_in;
   cc0_out     <= cc0_out_w;
   cc1_out     <= cc1_out_w;
   cc2_out     <= cc2_out_w;
   pix_en_out  <= pix_en_out_w;
   vs_out      <= vs_out_w;
   hs_out      <= hs_out_w;
end if;
end process;


rank : entity work.rank2d_top(rtl)
generic map(
        DATA_WIDTH_CH0     => DATA_WIDTH_CH0,
        DATA_WIDTH_CH1     => DATA_WIDTH_CH1,
        DATA_WIDTH_CH2     => DATA_WIDTH_CH2,
        DATA_WIDTH_FILTER  => DATA_WIDTH_FILTER,
        WINDOW_WIDTH       => WINDOW_WIDTH,
        WINDOW_HEIGHT      => WINDOW_HEIGHT,
        NEW_INPUTS         => NEW_INPUTS,
        MAX_HORIZONTAL_RES => MAX_HORIZONTAL_RES,
        Y_GENERATOR_TYPE   => Y_GENERATOR_TYPE,
        FAMILY             => FAMILY)
port map (
        clk       => clk,
        rst       => rst,
        ce        => '1',
        pix_en_in => pix_en_in_r,
        rank      => rank_r,
        cc0_in    => cc0_in_r,
        cc1_in    => cc1_in_r,
        cc2_in    => cc2_in_r,
        vs_in     => vs_in_r,
        hs_in     => hs_in_r,
        cc0_out   => cc0_out_w,
        cc1_out   => cc1_out_w,
        cc2_out   => cc2_out_w,
        pix_en_out=> pix_en_out_w,
        vs_out    => vs_out_w,
        hs_out    => hs_out_w
);



end rtl;